@@ -34,12 +34,12 @@ which may result from the use of athcool.
    http://members.jcom.home.ne.jp/jacobi/linux/softwares.html
 
   (Related links)
- * CoolOn Project [ written in Japanese ]
-	http://homepage2.nifty.com/coolon/
+ * CoolOn Project
+	http://coolon.o-ya.net/
  * pciutils
 	http://atrey.karlin.mff.cuni.cz/~mj/pciutils.shtml
  * Athlon Powersaving HOWTO
-	http://cip.uni-trier.de/nofftz/linux/Athlon-Powersaving-HOWTO.html
+	http://www.daniel.nofftz.net/linux/Athlon-Powersaving-HOWTO.html
  * LVCool
 	http://vcool.occludo.net/
 
@@ -67,7 +67,7 @@ which may result from the use of athcool.
 struct pci_filter filter;
 struct pci_access *pacc;
 struct device *scan_devices( void );
-word get_conf_word(struct device *d, unsigned int pos);
+LONG_t get_conf_word(struct device *d, unsigned int pos);
 
 static void print_title( void )
 {
@@ -123,6 +123,104 @@ static void usage( const char *package )
 #endif
 }
 
+LONG_t read_reg(struct pci_dev *dev, int tp, int addr)
+{
+  LONG_t regval = 0;
+
+  switch (tp) {
+  case TP_BYTE:
+    regval = (LONG_t)pci_read_byte( dev, addr );
+    break;
+  case TP_WORD:
+    regval = (LONG_t)pci_read_word( dev, addr );
+    break;
+  case TP_LONG:
+    regval = (LONG_t)pci_read_long( dev, addr );
+    break;
+  default:
+    printf(" error : unknown reg type \n");
+    break;
+  }
+
+  return regval;
+}
+
+int write_reg(struct pci_dev *dev, int tp, int addr, LONG_t regval_cur, LONG_t regval_new)
+{
+  int res = FALSE;
+#ifndef DISABLE_WRITE_REG
+  LONG_t regval_after;
+#endif
+
+  if (regval_cur != regval_new) {
+    switch (tp) {
+    case TP_BYTE:
+#ifndef DISABLE_WRITE_REG
+      res = pci_write_byte( dev, addr, (BYTE_t) regval_new );
+      regval_after = read_reg(dev, tp, addr);
+      if (regval_new != regval_after) {
+	printf( " failed!\n" );
+	printf( "\tAddress 0x%02X : (expected) 0x%02X -> (actual) 0x%02X\n", 
+		addr, regval_new, regval_after );
+	res = FALSE;
+      } else {
+#endif
+	printf( " done\n" );
+	printf( "\tAddress 0x%02X : 0x%02X -> 0x%02X\n", 
+		addr, regval_cur, regval_new );
+#ifndef DISABLE_WRITE_REG
+      }
+#endif
+      break;
+
+    case TP_WORD:
+#ifndef DISABLE_WRITE_REG
+      res = pci_write_word( dev, addr, (WORD_t)regval_new );
+      regval_after = read_reg(dev, tp, addr);
+      if (regval_new != regval_after) {
+	printf( " failed!\n" );
+	printf( "\tAddress 0x%02X - 0x%02X : (expected) 0x%04X -> (actual) 0x%04X\n", 
+		addr, addr+1, regval_new, regval_after );
+	res = FALSE;
+      } else {
+#endif
+	printf( " done\n" );
+	printf( "\tAddress 0x%02X - 0x%02X : 0x%04X -> 0x%04X\n", 
+		addr, addr+1, regval_cur, regval_new );
+#ifndef DISABLE_WRITE_REG
+      }
+#endif
+      break;
+
+    case TP_LONG:
+#ifndef DISABLE_WRITE_REG
+      res = pci_write_long( dev, addr, (LONG_t) regval_new );
+      regval_after = read_reg(dev, tp, addr);
+      if (regval_new != regval_after) {
+	printf( " failed!\n" );
+	printf( "\tAddress 0x%02X - 0x%02X : (expected) 0x%08X -> (actual) 0x%08X\n", 
+		addr, addr+3, regval_new, regval_after );
+	res = FALSE;
+      } else {
+#endif
+	printf( " done\n" );
+	printf( "\tAddress 0x%02X - 0x%02X : 0x%08X -> 0x%08X\n", 
+		addr, addr+3, regval_cur, regval_new );
+#ifndef DISABLE_WRITE_REG
+      }
+#endif
+      break;
+    default:
+      printf(" error : unknown reg type \n");
+      break;
+    }
+  } else {
+    printf(" already changed\n" );
+  }
+
+  return res;
+}
+
 int setAthlonPowersave( int m, int force_id, struct device *first_dev )
 {
   struct device	*d = NULL;
@@ -130,13 +228,9 @@ int setAthlonPowersave( int m, int force_id, struct device *first_dev )
   AthlonChips_s	*p_athlon = NULL;
   ChipReg_s	*p_reg;
   EachReg_s	*p_eachreg;
-  unsigned int	regval_cur, regval_new;
+  LONG_t	regval_cur, regval_new;
   int fixup = FALSE;
-  uint8_t addr;
-  uint8_t rev;
-#ifndef DISABLE_WRITE_REG
-  unsigned int res;
-#endif
+  int addr;
 
   if (m & ATHLONPM_FIXUP) {
     fixup = TRUE;
@@ -144,10 +238,10 @@ int setAthlonPowersave( int m, int force_id, struct device *first_dev )
   }
 
 #ifdef ENABLE_FORCEID
-  word class_id;
+  LONG_t class_id;
 
   if (force_id >= 0) {
-    /*    d = first_dev; */
+
     /* search Host Bridge device */
     for ( d = first_dev; d != NULL; d= d->next ){
       class_id = get_conf_word(d, PCI_CLASS_DEVICE);
@@ -166,6 +260,7 @@ int setAthlonPowersave( int m, int force_id, struct device *first_dev )
       printf( " Host Bridge is not found. exit. \n");
       exit (1);
     }
+
     /* search forced Reg ID */
     for ( p_reg = ChipReg; p_reg->regs->desc != NULL; p_reg++ ) {
       if ( p_reg->regid == force_id ) {	break; }
@@ -173,6 +268,7 @@ int setAthlonPowersave( int m, int force_id, struct device *first_dev )
     printf( " *** disabled auto detection on user request.\n" );
   } else {
 #endif
+
     /* search supported chipset */
     for ( d = first_dev; d != NULL; d = d->next ) {
       for ( p_athlon = AthlonChips; p_athlon->name != NULL ; p_athlon++ ) {
@@ -198,6 +294,7 @@ int setAthlonPowersave( int m, int force_id, struct device *first_dev )
 #ifdef ENABLE_FORCEID
   }
 #endif
+
   if ( p_reg->regs->desc == NULL ) {
     printf( "internal error."
 	    " supported Chipset found,"
@@ -207,18 +304,8 @@ int setAthlonPowersave( int m, int force_id, struct device *first_dev )
 
   /* read current value of register */
   for ( p_eachreg = p_reg->regs; p_eachreg->desc != NULL; p_eachreg++ ) {
-    switch (p_reg->tp) {
-    case BYTE:
-      regval_cur = pci_read_byte( d->dev, p_eachreg->addr );
-      break;
-    case SHORT:
-      regval_cur = pci_read_word( d->dev, p_eachreg->addr );
-      break;
-    case LONG:
-    default:
-      regval_cur = pci_read_long( d->dev, p_eachreg->addr );
-      break;
-    }
+
+    regval_cur = read_reg( d->dev, p_reg->tp, p_eachreg->addr );
 
     switch (m) {
     case ATHLONPM_STAT:
@@ -241,40 +328,10 @@ int setAthlonPowersave( int m, int force_id, struct device *first_dev )
 	break;
       }
 
-    //if ( m == ATHLONPM_OFF ) { regval_new = regval_cur ^ p_eachreg->bit; }
       if ( m == ATHLONPM_OFF ) { regval_new = regval_cur & ~p_eachreg->bit; }
 
       /* write new value into register */
-      switch (p_reg->tp) {
-      case BYTE:
-#ifndef DISABLE_WRITE_REG
-	res = pci_write_byte( d->dev, p_eachreg->addr, (uint8_t) regval_new );
-#endif
-	printf( " done\n" );
-	printf( "\tAddress 0x%02X : 0x%02X -> 0x%02X\n", 
-	      p_eachreg->addr, regval_cur, regval_new );
-	break;
-      case SHORT:
-#ifndef DISABLE_WRITE_REG
-	res = pci_write_word( d->dev, p_eachreg->addr, (uint16_t) regval_new );
-#endif
-	printf( " done\n" );
-	/* x86 cpus are little-endian, so values are inverted in registers */
-	printf( "\tAddress 0x%02X - 0x%02X : 0x%04X -> 0x%04X\n", 
-		p_eachreg->addr, p_eachreg->addr+1, regval_cur, regval_new );
-	break;
-      case LONG:
-      default:
-#ifndef DISABLE_WRITE_REG
-	res = pci_write_long( d->dev, p_eachreg->addr, (uint32_t) regval_new );
-#endif
-	printf( " done\n" );
-	/* x86 cpus are little-endian, so values are inverted in registers */
-	printf( "\tAddress 0x%02X - 0x%02X : 0x%08X -> 0x%08X\n", 
-		p_eachreg->addr, p_eachreg->addr+3, regval_cur, regval_new );
-	break;
-      }
-    
+      write_reg( d->dev, p_reg->tp, p_eachreg->addr, regval_cur, regval_new);
       break;
       /* end of enabling/disabling bits */
     }
@@ -284,58 +341,46 @@ int setAthlonPowersave( int m, int force_id, struct device *first_dev )
   if (fixup) {
     switch (p_reg->regid) {
 
+    case REGID_VIA_KT133:
+    case REGID_VIA_KT266:
+    case REGID_VIA_KT400:
       /* fix audio problem for VIA chipsets (0x70 bit 3) */
       /* from information provided by Wijatmoko U. Prayitno*/
       /* 0.3.5: enable bit3(Enhance CPU to PCI Write?),
 	               bit2(PCI Master Read Buffering),
 		       bit1(Delay Transaction) (=0x0E) on all VIA platform */
-    case REGID_VIA_KT133:
-    case REGID_VIA_KT266:
-    case REGID_VIA_KT400:
       addr = 0x70;
       regval_cur = pci_read_byte( d->dev, addr );
       regval_new = regval_cur | 0x0E;
       printf( "Fixup for VIA audio problem ... ");
-      if (regval_cur != regval_new) {
-#ifndef DISABLE_WRITE_REG
-	res = pci_write_byte( d->dev, addr, (uint8_t) regval_new );
-#endif
-	printf( "done.\n");
-	printf( "\tAddress 0x%02X : 0x%02X -> 0x%02X\n", 
-	      addr, regval_cur, regval_new );
-      } else {
-	printf( "already fixed.\n");
-      }
+
+      write_reg( d->dev, TP_BYTE, addr, regval_cur, regval_new);
       break;
 
+    case REGID_NVIDIA_NFORCE2:
       /* fixup for C1 HALT disconnect problem on nForce2 system */
       /* A hang is caused when the CPU generates a very fast CONNECT/HALT
 	 cycle sequence.  Workaround is to set the SYSTEM_IDLE_TIMEOUT to
 	 80 ns.
          'C18D' means nforce2 rev.C1 or greater */
+	//  Chip  Old Value  New Value
+	//  C17   0x1F0FFF01 0x1F01FF01
+	//  C18D  0x9F0FFF01 0x9F01FF01
       /* from information provided by Allen Martin. Thanks Herrmann. */
-    case REGID_NVIDIA_NFORCE2:
       if (m == ATHLONPM_ON) {
-	addr = 0x6c;
+	/*
+	addr = 0x6C;
 	rev = pci_read_byte(d->dev, PCI_REVISION_ID);
-	/* Chip  Old Value  New Value
-	   C17   0x1F0FFF01 0x1F01FF01
-	   C18D  0x9F0FFF01 0x9F01FF01
-	*/
-	//	regval_new = rev < 0xC1 ? 0x1F0FFF01 : 0x9F0FFF01;
 	regval_new = rev < 0xC1 ? 0x1F01FF01 : 0x9F01FF01;
 	regval_cur = pci_read_long(d->dev, addr);
 	printf( "Fixup for C1 HALT disconnect problem ... ");
-	if (regval_cur != regval_new) {
-#ifndef DISABLE_WRITE_REG
-	  res = pci_write_long(d->dev, addr, regval_new);
-#endif
-	  printf( "done.\n");
-	  printf( "\tAddress 0x%02X - 0x%02X : 0x%08X -> 0x%08X\n", 
-		  addr, addr+3, regval_cur, regval_new );
-	} else {
-	  printf( "already fixed.\n");
-	}
+	write_reg( d->dev, TP_LONG, addr, regval_cur, regval_new );
+	*/
+	addr = 0x6e;
+	regval_cur = pci_read_byte(d->dev, addr);
+	regval_new = 0x01;
+	printf( "Fixup for C1 HALT disconnect problem ... ");
+	write_reg( d->dev, TP_BYTE, addr, regval_cur, regval_new );
       } else {
 	printf( "nForce2 fixup is needed with 'on' option.\n");
       }
